use std::sync::Arc;

use eyre::Result;
use serde::{Deserialize, Serialize};

use crate::app_state::AppState;

pub struct ChatInstance {
    pub id: String,
    pub domain: String,
    pub base_url: String,
    pub public_key: String,
    pub status: InstanceStatus,
    pub reason: Option<String>,
}

#[derive(Serialize, Deserialize, Debug, sqlx::Type)]
#[sqlx(type_name = "instance_status", rename_all = "lowercase")]
pub enum InstanceStatus {
    Active,
    Suspended,
}

impl ChatInstance {
    pub async fn get(state: Arc<AppState>, domain: String) -> Result<Self> {
        if let Some(instance) = sqlx::query_as!(
            Self,
            r#"select id, domain, base_url, public_key,
            status as "status: InstanceStatus", reason
            from chat_instances where domain = $1"#,
            domain
        )
        .fetch_optional(&state.pool)
        .await?
        {
            return Ok(instance);
        }

        // TODO: identify server process
        // only try HTTP if `state.config.should_try_insecure()`

        todo!()
    }
}
